package com.redhat.server.mdb;

import java.util.Date;
import java.util.HashMap;

import javax.ejb.ActivationConfigProperty; 
import javax.ejb.MessageDriven; 
import javax.jms.JMSException; 
import javax.jms.Message; 
import javax.jms.MessageListener; 

//import javax.jms.Queue;
import org.apache.commons.lang.SerializationUtils;
import org.apache.log4j.Logger;
//import org.quartz.CronScheduleBuilder;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SimpleTrigger;
//import org.quartz.Trigger;
//import org.quartz.TriggerBuilder;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.triggers.SimpleTriggerImpl;

import com.redhat.messaging.MessageUtils;


@MessageDriven(name = "QuartzBean", activationConfig = { 
@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),  
@ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/requestQueue1"),  
@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge") }
) 
public class QuartzBean implements MessageListener 
{ 
	private static final Logger logger = Logger.getLogger(QuartzBean.class);
	
	public void onMessage(Message msg)
	{
		try {
			byte[] results = MessageUtils.getBytes(msg);
			
			@SuppressWarnings("unchecked")
			HashMap<String, Object> hashMap = (HashMap<String, Object>) SerializationUtils.deserialize(results);
			
			logger.info(hashMap.toString());
			//System.out.println(hashMap.toString());
			//hashMap.remove(CALL_DESTINATION);
			
			processMessage(hashMap);
			//notifyObservers(hashMap);
			
		} catch (JMSException e) {
			logger.warn("onMessage()", e);
		}  
	}
	
	protected void processMessage(HashMap<String, Object> hashMap) 
	{
		try {
			String processID = (String) hashMap.get("brms.processID");
			Long processInstanceID = (Long) hashMap.get("brms.processInstanceID");
			Integer sessionID = (Integer) hashMap.get("brms.sessionID");
			Long taskID = (Long) hashMap.get("brms.taskID");
			String taskName = (String) hashMap.get("brms.taskName");
			Long timeLimit = (Long) hashMap.get("brms.taskTimeLimit");
			String jobGroup = processID + "." + processInstanceID;
			String jobName = taskName +"."+ taskID;
			
			JobKey jobKeyA = new JobKey(jobName, jobGroup);
	    	JobDetail jobA = JobBuilder.newJob(JobA.class)
	    	.withIdentity(jobKeyA)
			.usingJobData("brms.processID", processID)
			.usingJobData("brms.processInstanceID", processInstanceID)
	        .usingJobData("brms.sessionID", sessionID)
	        .usingJobData("brms.taskID", taskID)
	        .usingJobData("brms.taskName", taskName)
			.build();
//
	    	long startTime = System.currentTimeMillis() + timeLimit;

	    	SimpleTrigger trigger1 = new SimpleTriggerImpl(jobName,
	    											   jobGroup,
	    	                                           new Date(startTime),
	    	                                           null,
	    	                                           0,
	    	                                           0L);
/*/ 
	    	Trigger trigger1 = TriggerBuilder
			.newTrigger()
			.withIdentity("triggerName1", "group1")
			.withSchedule(
				CronScheduleBuilder.cronSchedule("0/5 * * * * ?"))
			.build();
*/	 
	    	Scheduler scheduler = new StdSchedulerFactory().getScheduler();
	 
	    	scheduler.start();
	    	scheduler.scheduleJob(jobA, trigger1);

		} catch (Exception ex) {
			logger.warn("processMessage()", ex);
		}
	}
}